// -*-c++-*-

// fixg2sxd - a utility to convert fig to sxd format

// Copyright (C) 2003-2008 Alexander Bürger, acfb@users.sourceforge.net

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "xfigobjects.h"

istream& OpenCompound::read( istream& figfile )
{
    figfile >> upperleft_corner_x >> upperleft_corner_y
            >> lowerright_corner_x >> lowerright_corner_y;
    return figfile;
}

ostream& OpenCompound::write( ostream& out )
{
    out << "<draw:g>" << endl;
    return out;
}

istream& CloseCompound::read( istream& figfile )
{
    return figfile;
}

ostream& CloseCompound::write( ostream& out )
{
    out << "</draw:g>" << endl;
    return out;
}

